#include "settings.h"
#include "sockets.h"
#include "fileio.h"
#include <stdlib.h>
#include "math.h"
#include "clientprot.h"


//////// Settings macros ////////

Uint32 getPrivateProfile32(char * section, char * key, char * def, char * path)
{
	char buffer[32];

	GetPrivateProfileString(section, key, def, buffer, 32, path);

	return atoi(buffer);
}

Uint32 getSetting32(HKEY baseKey, const char * path, const char * value)
{
	// Open the key
	Uint32 key;			// Handle to a session with a registry key

	RegOpenKey((HKEY)baseKey, path, (HKEY*)&key);

	// Get a value
	DWORD buffer;		// Results of transaction will go here
	DWORD buflen = 4;	// Length of the buffer is 4, naturally
	DWORD type;			// Type will contain type of data transfered

	RegQueryValueEx((HKEY)key, value, NULL, &type, (BYTE*)&buffer, &buflen);

	// Close the key
	RegCloseKey((HKEY)key);

	// Check data for validity
	if (type == REG_DWORD)
		return buffer;
	else
		return -1;			// -1 = error
}

void setSetting32(HKEY baseKey, const char * path, const char * value, Uint32 buffer)
{
	// Open the key
	DWORD key;			// Handle to a session with a registry key

	RegCreateKey((HKEY)baseKey, path, (HKEY*)&key);

	// Get a value
	RegSetValueEx((HKEY)key, value, 0, REG_DWORD, (BYTE*)&buffer, 4);

	// Close the key
	RegCloseKey((HKEY)key);
}


//////// Operators ////////

opEntry::opEntry(char * nname, char * ppass, Operator_Level aaccess)
{
	setName(nname);
	setPassword(ppass);
	setAccess(aaccess);

	loginCount = 0;
	currentLogs = 0;
}

void opEntry::setPassword(char * ppass)
{
	plen = 0;

	memset(pass, 0, 32);
	while (ppass[plen++]) {}
	if (plen > 32) plen = 32;
	MEMCPY(ppass, pass, plen);
}

void opEntry::setName(char * nname)
{
	Uint16 i;
	nlen = 0;

	memset(name, 0, 20);
	while (nname[nlen++]) {}
	if (nlen > 20) nlen = 20;
	for (i = 0; i < nlen; i++)
		name[i] = tolower(nname[i]);
}

void opEntry::setAccess(Operator_Level aaccess)
{
	access = aaccess;
}

Operator_Level opEntry::getAccess()
{
	return access;
}

void opEntry::addCounter()
{
	loginCount++;
	currentLogs++;
}

void opEntry::removeCounter()
{
	currentLogs--;
}

Uint32 opEntry::getOverallCount()
{
	return loginCount;
}

Uint32 opEntry::getCurrentCount()
{
	return currentLogs;
}

bool opEntry::validateName(char * nname)
{
	Uint16 i,
		   len = 0;

	while (nname[len++]) {}

	if (len != nlen)
		return false;

	for (i = 0; i < len; i++)
	{
		if (name[i] != tolower(nname[i]))
			return false;
	}

	return true;
}

bool opEntry::validatePass(char * ppass)
{
	if (!*pass) return true;	// NULL passwords are unassigned

	Uint16 i,
		   len = 0;

	while (ppass[len++]) {}

	if (len != plen)
		return false;

	for (i = 0; i < len; i++)
	{
		if (pass[i] != ppass[i])
			return false;
	}

	return true;
}

opEntry * BOT_DATABASE::findOperator(char * name)
{
	_listnode <opEntry> * parse = opList.head;

	while (parse)
	{
		opEntry * op = parse->item;

		if (op->validateName(name))
		{
			return op;
		}

		parse = parse->next;
	}

	return NULL;
}

opEntry * BOT_DATABASE::addOperator(char * name)
{
	opEntry * op;

	op = findOperator(name);
	if (op == NULL)
	{
		op = new opEntry(name, "", OP_Moderator);
		if (op) opList.prepend(op);
	}

	return op;
}

opEntry * BOT_DATABASE::addOperator(char * name, char * pass)
{
	opEntry * op = addOperator(name);

	op->setPassword(pass);

	return op;
}

opEntry * BOT_DATABASE::addOperator(char * name, Operator_Level level)
{
	opEntry * op = addOperator(name);

	op->setAccess(level);

	return op;
}

opEntry * BOT_DATABASE::addOperator(char * name, char * pass, Operator_Level level)
{
	opEntry * op = addOperator(name);

	op->setPassword(pass);
	op->setAccess(level);

	return op;
}

bool BOT_DATABASE::removeOperator(char * name)
{
	opEntry * op = findOperator(name);
	if (op == NULL) return false;

	opList.kill(op, false);
	return true;
}


//////// Bot descriptor ////////

BOT_INFO::BOT_INFO()
{
	setLogin("Bobo the Bot", "+++++", resolveHostname("127.0.0.1"), 2000);
	setArena("@0", SHIP_Spectator, 1024, 768);
	resetSystemInfo();	// Ban info
	setReg("Catid.bot", "Catid@pacbell.net", "CatVille", "CatState", SEX_Male, 17, true, true, true);
	setDatabase(NULL);
}

void BOT_INFO::set(BOT_INFO a)
{
	setLogin(a.name, a.password, a.ip, a.port);
	setArena(a.initialArena, a.initialShip, a.xres, a.yres);
	setBan(a.machineID, a.timeZoneBias, a.permissionID, a.clientVersion, a.processor, a.regName);
	setReg(a.realName, a.email, a.city, a.state, a.sex, a.age, a.playAtHome, a.playAtWork, a.playAtSchool);
	setDatabase(a.db);
}

void BOT_INFO::resetSystemInfo()
{
	// SubSpace version
	clientVersion = CLIENT_VERSION;

	// CPU type
	SYSTEM_INFO si;
	GetSystemInfo(&si);
	processor = si.dwProcessorType;

	// Windows registration name
	{
		// Get platform
		OSVERSIONINFO osvi;
		osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		GetVersionEx(&osvi);
		Uint32 PlatformId = osvi.dwPlatformId;

		// Prepare for registry access
		Uint32 key;			// Handle to a session with a registry key
		Uint32 buflen = 80;	// Length of the buffer
		Uint32 type;		// Type will contain type of data transfered
		memset(regName, 0, 80);

		if (PlatformId != VER_PLATFORM_WIN32_NT)
		{
			// Look up Windows 9x or Windows 3.1 registration name
			RegOpenKey(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion", (HKEY*)&key);
		}
		else
		{
			// Look up Windows NT registration name
			RegOpenKey(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", (HKEY*)&key);
		}

		RegQueryValueEx((HKEY)key, "RegisteredOwner", NULL, &type, (BYTE*)&regName, &buflen);

		RegCloseKey((HKEY)key);
	}

	// Machine ID
	GetVolumeInformation("C:\\", NULL, 0, &machineID, NULL, NULL, NULL, 0);
	if (machineID > 0x7fffffff) machineID += 0x7fffffff;

	// Timezone Bias
	TIME_ZONE_INFORMATION tzi;
	GetTimeZoneInformation(&tzi);
	timeZoneBias = (SHORT)tzi.Bias;

	// Permission ID
	permissionID = getSetting32(HKEY_LOCAL_MACHINE, "SOFTWARE", "D2");

	// Install some SubSpace registry keys
	setSetting32(HKEY_LOCAL_MACHINE, "SOFTWARE", "D1", machineID);
	if (permissionID == -1)
	{
		permissionID = (GetTickCount() ^ 0xAAAAAAAA) * 0x5f346d + 0x5abcdef;
		setSetting32(HKEY_LOCAL_MACHINE, "SOFTWARE", "D2", permissionID);
	}
}

void BOT_INFO::setLogin(char * nname, char * ppassword, Uint32 iip, Uint16 pport)
{
	Uint32 len;

	len = limit(strlen(nname), 19);
	MEMCPY(nname, name, len);
	name[len] = 0;

	len = limit(strlen(ppassword), 31);
	MEMCPY(ppassword, password, len);
	password[len] = 0;

	ip = iip;
	port = pport;
}

void BOT_INFO::setDatabase(BOT_DATABASE * ddb)
{
	db = ddb;
}

void BOT_INFO::setArena(char * iinitialArena, Ship_Types iinitialShip, Uint16 xxres, Uint16 yyres)
{
	Uint32 len;

	len = limit(strlen(iinitialArena), 15);
	MEMCPY(iinitialArena, initialArena, len);
	initialArena[len] = 0;

	initialShip = iinitialShip;

	xres = xxres;
	yres = yyres;
}

void BOT_INFO::setBan(Uint32 mmachineID, Uint16 ttimeZoneBias, Uint32 ppermissionID, Uint16 cclientVersion, Uint32 pprocessor, char * rregName)
{
	Uint32 len;

	machineID = mmachineID;
	timeZoneBias = ttimeZoneBias;
	permissionID = ppermissionID;
	clientVersion = cclientVersion;
	processor = pprocessor;

	len = limit(strlen(rregName), 79);
	MEMCPY(rregName, regName, len);
	regName[len] = 0;
}

void BOT_INFO::setReg(char * rrealName, char * eemail, char * ccity, char * sstate, RegForm_Sex ssex, BYTE aage, bool pplayAtHome, bool pplayAtWork, bool pplayAtSchool)
{
	Uint32 len;

	len = limit(strlen(rrealName), 31);
	MEMCPY(rrealName, realName, len);
	realName[len] = 0;

	len = limit(strlen(eemail), 63);
	MEMCPY(eemail, email, len);
	email[len] = 0;

	len = limit(strlen(ccity), 31);
	MEMCPY(ccity, city, len);
	city[len] = 0;

	len = limit(strlen(sstate), 23);
	MEMCPY(sstate, state, len);
	state[len] = 0;

	sex = ssex;
	age = aage;
	playAtHome = pplayAtHome;
	playAtWork = pplayAtWork;
	playAtSchool = pplayAtSchool;
}
